@@ -3,7 +3,7 @@
 #
 # kluster/kox point process models
 #
-# $Revision: 1.158 $ $Date: 2020/11/16 01:32:06 $
+# $Revision: 1.159 $ $Date: 2021/04/15 04:25:16 $
 #
 
 kppm <- function(X, ...) {
@@ -1504,173 +1504,6 @@ residuals.kppm <- residuals.dppm <- function(object, ...) {
   residuals(as.ppm(object), ...)
 }
 
-simulate.kppm <- function(object, nsim=1, seed=NULL, ...,
-                          window=NULL, covariates=NULL,
-                          verbose=TRUE, retry=10,
-                          drop=FALSE) {
-  starttime <- proc.time()
-  verbose <- verbose && (nsim > 1)
-  check.1.real(retry)
-  # .... copied from simulate.lm ....
-  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
-    runif(1)
-  if (is.null(seed))
-    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
-  else {
-    R.seed <- get(".Random.seed", envir = .GlobalEnv)
-    set.seed(seed)
-    RNGstate <- structure(seed, kind = as.list(RNGkind()))
-    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
-  }
-  
-  # ..................................
-  # determine window for simulation results
-  if(!is.null(window)) {
-    stopifnot(is.owin(window))
-    win <- window
-  } else {
-    win <- as.owin(object)
-  }
-  # ..................................
-  # determine parameters
-  mp <- as.list(object$modelpar)
-
-  # parameter 'mu'
-  # = parent intensity of cluster process
-  # = mean log intensity of log-Gaussian Cox process
-  
-  if(is.null(covariates) && (object$stationary || is.null(window))) {
-    # use existing 'mu' (scalar or image)
-    mu <- object$mu
-  } else {
-    # recompute 'mu' using new data
-    switch(object$clusters,
-           Cauchy=,
-           VarGamma=,
-           Thomas=,
-           MatClust={
-             # Poisson cluster process
-             kappa <- mp$kappa
-             lambda <- predict(object, window=win, covariates=covariates)
-             mu <- eval.im(lambda/kappa)
-           },
-           LGCP={
-             # log-Gaussian Cox process
-             sigma2 <- mp$sigma2
-             lambda <- predict(object, window=win, covariates=covariates)
-             mu <- eval.im(log(lambda) - sigma2/2)
-           },
-           stop(paste("Simulation of", sQuote(object$clusters),
-                      "processes is not yet implemented"))
-           )
-  }
-
-  # prepare data for execution
-  out <- list()
-  switch(object$clusters,
-         Thomas={
-           kappa <- mp$kappa
-           sigma <- mp$sigma
-           cmd <- expression(rThomas(kappa,sigma,mu,win, ...))
-           dont.complain.about(kappa, sigma, mu)
-         },
-         MatClust={
-           kappa <- mp$kappa
-           r     <- mp$R
-           cmd   <- expression(rMatClust(kappa,r,mu,win, ...))
-           dont.complain.about(kappa, r)
-         },
-         Cauchy = {
-           kappa <- mp$kappa
-           omega <- mp$omega
-           cmd   <- expression(rCauchy(kappa, omega, mu, win, ...))
-           dont.complain.about(kappa, omega, mu)
-         },
-         VarGamma = {
-           kappa  <- mp$kappa
-           omega  <- mp$omega
-           nu.ker <- object$covmodel$margs$nu.ker
-           cmd    <- expression(rVarGamma(kappa, nu.ker, omega, mu, win, ...))
-           dont.complain.about(kappa, nu.ker, omega, mu)
-         },
-         LGCP={
-           sigma2 <- mp$sigma2
-           alpha  <- mp$alpha
-           cm <- object$covmodel
-           model <- cm$model
-           margs <- cm$margs
-           param <- append(list(var=sigma2, scale=alpha), margs)
-           #' 
-           if(!is.im(mu)) {
-             # model will be simulated in 'win'
-             cmd <- expression(rLGCP(model=model, mu=mu, param=param,
-                               ..., win=win))
-             #' check that RandomFields package recognises parameter format
-             rfmod <- try(rLGCP(model, mu=mu, param=param, win=win,
-                              ..., modelonly=TRUE))
-           } else {
-             # model will be simulated in as.owin(mu), then change window
-             cmd <- expression(rLGCP(model=model, mu=mu, param=param,
-                               ...)[win])
-             #' check that RandomFields package recognises parameter format
-             rfmod <- try(rLGCP(model, mu=mu, param=param, 
-                              ..., modelonly=TRUE))
-           }
-           #' suppress warnings from code checker
-           dont.complain.about(model, mu, param)
-           #' check that model is recognised
-           if(inherits(rfmod, "try-error"))
-             stop(paste("Internal error in simulate.kppm:",
-                        "unable to build Random Fields model",
-                        "for log-Gaussian Cox process"))
-         })
-  
-  # run
-  if(verbose) {
-    cat(paste("Generating", nsim, "simulations... "))
-    state <- list()
-  }
-  for(i in 1:nsim) {
-    out[[i]] <- try(eval(cmd))
-    if(verbose) state <- progressreport(i, nsim, state=state)
-  }
-  # detect failures
-  if(any(bad <- unlist(lapply(out, inherits, what="try-error")))) {
-    nbad <- sum(bad)
-    gripe <- paste(nbad,
-                   ngettext(nbad, "simulation was", "simulations were"),
-                   "unsuccessful")
-    if(verbose) splat(gripe)
-    if(retry <= 0) {
-      fate <- "returned as NULL"
-      out[bad] <- list(NULL)
-    } else {
-      if(verbose) cat("Retrying...")
-      ntried <- 0
-      while(ntried < retry) {
-        ntried <- ntried + 1
-        for(j in which(bad))
-          out[[j]] <- try(eval(cmd))
-        bad <- unlist(lapply(out, inherits, what="try-error"))
-        nbad <- sum(bad)
-        if(nbad == 0) break
-      }
-      if(verbose) cat("Done.\n")
-      fate <- if(nbad == 0) "all recomputed" else
-              paste(nbad, "simulations still unsuccessful")
-      fate <- paste(fate, "after", ntried,
-                    ngettext(ntried, "further try", "further tries"))
-    }
-    warning(paste(gripe, fate, sep=": "))
-  }
-  if(verbose)
-    cat("Done.\n")
-  #' pack up
-  out <- simulationresult(out, nsim, drop)
-  out <- timed(out, starttime=starttime)
-  attr(out, "seed") <- RNGstate
-  return(out)
-}
 
 formula.kppm <- formula.dppm <- function(x, ...) {
   formula(x$po, ...)
